@@ -76,6 +76,7 @@ typedef struct
 
 static gchar **sourcedirs = NULL;
 static gchar *xmllint = NULL;
+static gchar *jsonformat = NULL;
 static gchar *gdk_pixbuf_pixdata = NULL;
 
 static void
@@ -219,7 +220,6 @@ end_element (GMarkupParseContext  *context,
       gchar *key;
       FileData *data = NULL;
       char *tmp_file = NULL;
-      char *tmp_file2 = NULL;
 
       file = state->string->str;
       key = file;
@@ -274,6 +274,7 @@ end_element (GMarkupParseContext  *context,
           gchar **options;
           guint i;
           gboolean xml_stripblanks = FALSE;
+          gboolean json_stripblanks = FALSE;
           gboolean to_pixdata = FALSE;
 
           options = g_strsplit (state->preproc_options, ",", -1);
@@ -284,6 +285,8 @@ end_element (GMarkupParseContext  *context,
                 xml_stripblanks = TRUE;
               else if (!strcmp (options[i], "to-pixdata"))
                 to_pixdata = TRUE;
+              else if (!strcmp (options[i], "json-stripblanks"))
+                json_stripblanks = TRUE;
               else
                 {
                   g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
@@ -294,73 +297,148 @@ end_element (GMarkupParseContext  *context,
             }
           g_strfreev (options);
 
-          if (xml_stripblanks && xmllint != NULL)
+          if (xml_stripblanks)
             {
-              int fd;
-	      GSubprocess *proc;
-
-              tmp_file = g_strdup ("resource-XXXXXXXX");
-              if ((fd = g_mkstemp (tmp_file)) == -1)
+              /* This is not fatal: pretty-printed XML is still valid XML */
+              if (xmllint == NULL)
                 {
-                  int errsv = errno;
+                  static gboolean xmllint_warned = FALSE;
 
-                  g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errsv),
-                               _("Failed to create temp file: %s"),
-                              g_strerror (errsv));
-                  g_free (tmp_file);
-                  tmp_file = NULL;
-                  goto cleanup;
+                  if (!xmllint_warned)
+                    {
+                      /* Translators: the first %s is a gresource XML attribute,
+                       * the second %s is an environment variable, and the third
+                       * %s is a command line tool
+                       */
+                      char *warn = g_strdup_printf (_("%s preprocessing requested, but %s is not set, and %s is not in PATH"),
+                                                    "xml-stripblanks",
+                                                    "XMLLINT",
+                                                    "xmllint");
+                      g_printerr ("%s\n", warn);
+                      g_free (warn);
+
+                      /* Only warn once */
+                      xmllint_warned = TRUE;
+                    }
                 }
-              close (fd);
+              else
+                {
+                  GSubprocess *proc;
+                  int fd;
 
-              proc = g_subprocess_new (G_SUBPROCESS_FLAGS_STDOUT_SILENCE, error,
-                                       xmllint, "--nonet", "--noblanks", "--output", tmp_file, real_file, NULL);
-              g_free (real_file);
-	      real_file = NULL;
+                  fd = g_file_open_tmp ("resource-XXXXXXXX", &tmp_file, error);
+                  if (fd < 0)
+                    goto cleanup;
 
-	      if (!proc)
-		goto cleanup;
+                  close (fd);
 
-	      if (!g_subprocess_wait_check (proc, NULL, error))
-		{
-		  g_object_unref (proc);
-                  goto cleanup;
+                  proc = g_subprocess_new (G_SUBPROCESS_FLAGS_STDOUT_SILENCE, error,
+                                           xmllint, "--nonet", "--noblanks", "--output", tmp_file, real_file, NULL);
+                  g_free (real_file);
+                  real_file = NULL;
+
+                  if (!proc)
+                    goto cleanup;
+
+                  if (!g_subprocess_wait_check (proc, NULL, error))
+                    {
+                      g_object_unref (proc);
+                      goto cleanup;
+                    }
+
+                  g_object_unref (proc);
+
+                  real_file = g_strdup (tmp_file);
                 }
+            }
 
-	      g_object_unref (proc);
+          if (json_stripblanks)
+            {
+              /* As above, this is not fatal: pretty-printed JSON is still
+               * valid JSON
+               */
+              if (jsonformat == NULL)
+                {
+                  static gboolean jsonformat_warned = FALSE;
 
-              real_file = g_strdup (tmp_file);
+                  if (!jsonformat_warned)
+                    {
+                      /* Translators: the first %s is a gresource XML attribute,
+                       * the second %s is an environment variable, and the third
+                       * %s is a command line tool
+                       */
+                      char *warn = g_strdup_printf (_("%s preprocessing requested, but %s is not set, and %s is not in PATH"),
+                                                    "json-stripblanks",
+                                                    "JSON_GLIB_FORMAT",
+                                                    "json-glib-format");
+                      g_printerr ("%s\n", warn);
+                      g_free (warn);
+
+                      /* Only warn once */
+                      jsonformat_warned = TRUE;
+                    }
+                }
+              else
+                {
+                  GSubprocess *proc;
+                  int fd;
+
+                  fd = g_file_open_tmp ("resource-XXXXXXXX", &tmp_file, error);
+                  if (fd < 0)
+                    goto cleanup;
+
+                  close (fd);
+
+                  proc = g_subprocess_new (G_SUBPROCESS_FLAGS_STDOUT_SILENCE, error,
+                                           jsonformat, "--output", tmp_file, real_file, NULL);
+                  g_free (real_file);
+                  real_file = NULL;
+
+                  if (!proc)
+                    goto cleanup;
+
+                  if (!g_subprocess_wait_check (proc, NULL, error))
+                    {
+                      g_object_unref (proc);
+                      goto cleanup;
+                    }
+
+                  g_object_unref (proc);
+
+                  real_file = g_strdup (tmp_file);
+                }
             }
 
           if (to_pixdata)
             {
-              int fd;
 	      GSubprocess *proc;
+              int fd;
 
+              /* This is a fatal error: if to-pixdata is used it means that
+               * the code loading the GResource expects a specific data format
+               */
               if (gdk_pixbuf_pixdata == NULL)
                 {
-                  g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                                       "to-pixbuf preprocessing requested but GDK_PIXBUF_PIXDATA "
-                                       "not set and gdk-pixbuf-pixdata not found in path");
+                  /* Translators: the first %s is a gresource XML attribute,
+                   * the second %s is an environment variable, and the third
+                   * %s is a command line tool
+                   */
+                  g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                               _("%s preprocessing requested, but %s is not set, and %s is not in PATH"),
+                               "to-pixdata",
+                               "GDK_PIXBUF_PIXDATA",
+                               "gdk-pixbuf-pixdata");
                   goto cleanup;
                 }
 
-              tmp_file2 = g_strdup ("resource-XXXXXXXX");
-              if ((fd = g_mkstemp (tmp_file2)) == -1)
-                {
-                  int errsv = errno;
+              fd = g_file_open_tmp ("resource-XXXXXXXX", &tmp_file, error);
+              if (fd < 0)
+                goto cleanup;
 
-                  g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errsv),
-                               _("Failed to create temp file: %s"),
-			       g_strerror (errsv));
-                  g_free (tmp_file2);
-                  tmp_file2 = NULL;
-                  goto cleanup;
-                }
               close (fd);
 
               proc = g_subprocess_new (G_SUBPROCESS_FLAGS_STDOUT_SILENCE, error,
-                                       gdk_pixbuf_pixdata, real_file, tmp_file2, NULL);
+                                       gdk_pixbuf_pixdata, real_file, tmp_file, NULL);
               g_free (real_file);
               real_file = NULL;
 
@@ -372,7 +450,7 @@ end_element (GMarkupParseContext  *context,
 
 	      g_object_unref (proc);
 
-              real_file = g_strdup (tmp_file2);
+              real_file = g_strdup (tmp_file);
             }
 	}
 
@@ -437,12 +515,6 @@ done:
           g_free (tmp_file);
         }
 
-      if (tmp_file2)
-        {
-          unlink (tmp_file2);
-          g_free (tmp_file2);
-        }
-
       if (data != NULL)
         file_data_free (data);
     }
@@ -733,8 +805,10 @@ main (int argc, char **argv)
   xmllint = g_strdup (g_getenv ("XMLLINT"));
   if (xmllint == NULL)
     xmllint = g_find_program_in_path ("xmllint");
-  if (xmllint == NULL)
-    g_printerr ("XMLLINT not set and xmllint not found in path; skipping xml preprocessing.\n");
+
+  jsonformat = g_strdup (g_getenv ("JSON_GLIB_FORMAT"));
+  if (jsonformat == NULL)
+    jsonformat = g_find_program_in_path ("json-glib-format");
 
   gdk_pixbuf_pixdata = g_strdup (g_getenv ("GDK_PIXBUF_PIXDATA"));
   if (gdk_pixbuf_pixdata == NULL)
@@ -789,6 +863,7 @@ main (int argc, char **argv)
     {
       g_free (target);
       g_free (c_name);
+      g_hash_table_unref (files);
       return 1;
     }
 
@@ -858,6 +933,7 @@ main (int argc, char **argv)
               g_string_free (dep_string, TRUE);
               g_free (dependency_file);
               g_error_free (error);
+              g_hash_table_unref (files);
               return 1;
             }
         }
@@ -890,6 +966,7 @@ main (int argc, char **argv)
 	    {
 	      g_printerr ("Can't open temp file\n");
 	      g_free (c_name);
+              g_hash_table_unref (files);
 	      return 1;
 	    }
 	  close (fd);
@@ -936,6 +1013,7 @@ main (int argc, char **argv)
       g_printerr ("%s\n", error->message);
       g_free (target);
       g_free (c_name);
+      g_hash_table_unref (files);
       return 1;
     }
 
@@ -948,6 +1026,7 @@ main (int argc, char **argv)
 	{
 	  g_printerr ("can't write to file %s", target);
 	  g_free (c_name);
+          g_hash_table_unref (files);
 	  return 1;
 	}
 
@@ -985,6 +1064,7 @@ main (int argc, char **argv)
 	{
 	  g_printerr ("can't read back temporary file");
 	  g_free (c_name);
+          g_hash_table_unref (files);
 	  return 1;
 	}
       g_unlink (binary_target);
@@ -994,6 +1074,7 @@ main (int argc, char **argv)
 	{
 	  g_printerr ("can't write to file %s", target);
 	  g_free (c_name);
+          g_hash_table_unref (files);
 	  return 1;
 	}
 
@@ -1089,7 +1170,9 @@ main (int argc, char **argv)
   g_free (target);
   g_hash_table_destroy (table);
   g_free (xmllint);
+  g_free (jsonformat);
   g_free (c_name);
+  g_hash_table_unref (files);
 
   return 0;
 }
