@@ -92,18 +92,18 @@ enum_state_add_value (EnumState    *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "nick must be a minimum of 2 characters");
+                   _("nick must be a minimum of 2 characters"));
       return;
     }
 
   value = g_ascii_strtoll (valuestr, &end, 0);
-  if (*end || state->is_flags ?
+  if (*end || (state->is_flags ?
                 (value > G_MAXUINT32 || value < 0) :
-                (value > G_MAXINT32 || value < G_MININT32))
+                (value > G_MAXINT32 || value < G_MININT32)))
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "invalid numeric value");
+                   _("Invalid numeric value"));
       return;
     }
 
@@ -111,7 +111,7 @@ enum_state_add_value (EnumState    *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "<value nick='%s'/> already specified", nick);
+                   _("<value nick='%s'/> already specified"), nick);
       return;
     }
 
@@ -119,7 +119,7 @@ enum_state_add_value (EnumState    *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "value='%s' already specified", valuestr);
+                   _("value='%s' already specified"), valuestr);
       return;
     }
 
@@ -133,7 +133,7 @@ enum_state_add_value (EnumState    *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "flags values must have at most 1 bit set");
+                   _("flags values must have at most 1 bit set"));
       return;
     }
 
@@ -158,7 +158,7 @@ enum_state_end (EnumState **state_ptr,
   if (state->strinfo->len == 0)
     g_set_error (error,
                  G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                 "<%s> must contain at least one <value>",
+                 _("<%s> must contain at least one <value>"),
                  state->is_flags ? "flags" : "enum");
 }
 
@@ -312,8 +312,8 @@ key_state_check_range (KeyState  *state,
             {
               g_set_error (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
-                           "<%s> is not contained in "
-                           "the specified range", tag);
+                           _("<%s> is not contained in "
+                           "the specified range"), tag);
             }
         }
 
@@ -324,20 +324,20 @@ key_state_check_range (KeyState  *state,
               if (state->is_enum)
                 g_set_error (error, G_MARKUP_ERROR,
                              G_MARKUP_ERROR_INVALID_CONTENT,
-                             "<%s> is not a valid member of "
-                             "the specified enumerated type", tag);
+                             _("<%s> is not a valid member of "
+                             "the specified enumerated type"), tag);
 
               else if (state->is_flags)
                 g_set_error (error, G_MARKUP_ERROR,
                              G_MARKUP_ERROR_INVALID_CONTENT,
-                             "<%s> contains string not in the "
-                             "specified flags type", tag);
+                             _("<%s> contains string not in the "
+                             "specified flags type"), tag);
 
               else
                 g_set_error (error, G_MARKUP_ERROR,
                              G_MARKUP_ERROR_INVALID_CONTENT,
-                             "<%s> contains string not in "
-                             "<choices>", tag);
+                             _("<%s> contains a string not in "
+                             "<choices>"), tag);
             }
         }
     }
@@ -370,7 +370,7 @@ key_state_set_range (KeyState     *state,
     {
       g_set_error_literal (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
-                           "<range/> already specified for this key");
+                           _("<range/> already specified for this key"));
       return;
     }
 
@@ -388,7 +388,7 @@ key_state_set_range (KeyState     *state,
       gchar *type = g_variant_type_dup_string (state->type);
       g_set_error (error, G_MARKUP_ERROR,
                   G_MARKUP_ERROR_INVALID_CONTENT,
-                  "<range> not allowed for keys of type '%s'", type);
+                  _("<range> not allowed for keys of type “%s”"), type);
       g_free (type);
       return;
     }
@@ -405,7 +405,7 @@ key_state_set_range (KeyState     *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "<range> specified minimum is greater than maxmimum");
+                   _("<range> specified minimum is greater than maximum"));
       return;
     }
 
@@ -430,7 +430,7 @@ key_state_start_default (KeyState     *state,
         {
           g_set_error (error, G_MARKUP_ERROR,
                        G_MARKUP_ERROR_INVALID_CONTENT,
-                       "unsupported l10n category: %s", l10n);
+                       _("unsupported l10n category: %s"), l10n);
           return NULL;
         }
 
@@ -438,8 +438,8 @@ key_state_start_default (KeyState     *state,
         {
           g_set_error_literal (error, G_MARKUP_ERROR,
                                G_MARKUP_ERROR_INVALID_CONTENT,
-                               "l10n requested, but no "
-                               "gettext domain given");
+                               _("l10n requested, but no "
+                               "gettext domain given"));
           return NULL;
         }
 
@@ -450,8 +450,8 @@ key_state_start_default (KeyState     *state,
     {
       g_set_error_literal (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
-                           "translation context given for "
-                           " value without l10n enabled");
+                           _("translation context given for "
+                           "value without l10n enabled"));
       return NULL;
     }
 
@@ -472,7 +472,7 @@ key_state_end_default (KeyState  *state,
   if (!state->default_value)
     {
       gchar *type = g_variant_type_dup_string (state->type);
-      g_prefix_error (error, "failed to parse <default> value of type '%s': ", type);
+      g_prefix_error (error, _("Failed to parse <default> value of type “%s”: "), type);
       g_free (type);
     }
 
@@ -489,8 +489,8 @@ key_state_start_choices (KeyState  *state,
     {
       g_set_error_literal (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
-                           "<choices> cannot be specified for keys "
-                           "tagged as having an enumerated type");
+                           _("<choices> cannot be specified for keys "
+                           "tagged as having an enumerated type"));
       return;
     }
 
@@ -498,7 +498,7 @@ key_state_start_choices (KeyState  *state,
     {
       g_set_error_literal (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
-                           "<choices> already specified for this key");
+                           _("<choices> already specified for this key"));
       return;
     }
 
@@ -510,7 +510,7 @@ key_state_start_choices (KeyState  *state,
       gchar *type_string = g_variant_type_dup_string (state->type);
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "<choices> not allowed for keys of type '%s'",
+                   _("<choices> not allowed for keys of type “%s”"),
                    type_string);
       g_free (type_string);
       return;
@@ -526,7 +526,7 @@ key_state_add_choice (KeyState     *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "<choice value='%s'/> already given", choice);
+                   _("<choice value='%s'/> already given"), choice);
       return;
     }
 
@@ -541,7 +541,7 @@ key_state_end_choices (KeyState  *state,
   if (!state->has_choices)
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                   "<choices> must contain at least one <choice>");
+                   _("<choices> must contain at least one <choice>"));
       return;
     }
 
@@ -555,12 +555,12 @@ key_state_start_aliases (KeyState  *state,
   if (state->has_aliases)
     g_set_error_literal (error, G_MARKUP_ERROR,
                          G_MARKUP_ERROR_INVALID_CONTENT,
-                         "<aliases> already specified for this key");
+                         _("<aliases> already specified for this key"));
   else if (!state->is_flags && !state->is_enum && !state->has_choices)
     g_set_error_literal (error, G_MARKUP_ERROR,
                          G_MARKUP_ERROR_INVALID_CONTENT,
-                         "<aliases> can only be specified for keys with "
-                         "enumerated or flags types or after <choices>");
+                         _("<aliases> can only be specified for keys with "
+                         "enumerated or flags types or after <choices>"));
 }
 
 static void
@@ -578,21 +578,21 @@ key_state_add_alias (KeyState     *state,
           if (state->is_enum)
             g_set_error (error, G_MARKUP_ERROR,
                          G_MARKUP_ERROR_INVALID_CONTENT,
-                         "<alias value='%s'/> given when '%s' is already "
-                         "a member of the enumerated type", alias, alias);
+                         _("<alias value='%s'/> given when “%s” is already "
+                         "a member of the enumerated type"), alias, alias);
 
           else
             g_set_error (error, G_MARKUP_ERROR,
                          G_MARKUP_ERROR_INVALID_CONTENT,
-                         "<alias value='%s'/> given when "
-                         "<choice value='%s'/> was already given",
+                         _("<alias value='%s'/> given when "
+                         "<choice value='%s'/> was already given"),
                          alias, alias);
         }
 
       else
         g_set_error (error, G_MARKUP_ERROR,
                      G_MARKUP_ERROR_INVALID_CONTENT,
-                     "<alias value='%s'/> already specified", alias);
+                     _("<alias value='%s'/> already specified"), alias);
 
       return;
     }
@@ -601,8 +601,10 @@ key_state_add_alias (KeyState     *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "alias target '%s' is not in %s", target,
-                   state->is_enum ? "enumerated type" : "<choices>");
+                   state->is_enum ?
+                     _("alias target “%s” is not in enumerated type") :
+                     _("alias target “%s” is not in <choices>"),
+                   target);
       return;
     }
 
@@ -616,7 +618,7 @@ key_state_end_aliases (KeyState  *state,
   if (!state->has_aliases)
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                   "<aliases> must contain at least one <alias>");
+                   _("<aliases> must contain at least one <alias>"));
       return;
     }
 }
@@ -743,6 +745,8 @@ key_state_free (gpointer data)
 {
   KeyState *state = data;
 
+  g_free (state->child_schema);
+
   if (state->type)
     g_variant_type_free (state->type);
 
@@ -781,7 +785,7 @@ is_valid_keyname (const gchar  *key,
   if (key[0] == '\0')
     {
       g_set_error_literal (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                           _("empty names are not permitted"));
+                           _("Empty names are not permitted"));
       return FALSE;
     }
 
@@ -791,7 +795,7 @@ is_valid_keyname (const gchar  *key,
   if (!g_ascii_islower (key[0]))
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                   _("invalid name '%s': names must begin "
+                   _("Invalid name “%s”: names must begin "
                      "with a lowercase letter"), key);
       return FALSE;
     }
@@ -803,17 +807,17 @@ is_valid_keyname (const gchar  *key,
           !g_ascii_isdigit (key[i]))
         {
           g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                       _("invalid name '%s': invalid character '%c'; "
-                         "only lowercase letters, numbers and hyphen ('-') "
-                         "are permitted."), key, key[i]);
+                       _("Invalid name “%s”: invalid character “%c”; "
+                         "only lowercase letters, numbers and hyphen (“-”) "
+                         "are permitted"), key, key[i]);
           return FALSE;
         }
 
       if (key[i] == '-' && key[i + 1] == '-')
         {
           g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                       _("invalid name '%s': two successive hyphens ('--') "
-                         "are not permitted."), key);
+                       _("Invalid name “%s”: two successive hyphens (“--”) "
+                         "are not permitted"), key);
           return FALSE;
         }
     }
@@ -821,15 +825,15 @@ is_valid_keyname (const gchar  *key,
   if (key[i - 1] == '-')
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                   _("invalid name '%s': the last character may not be a "
-                     "hyphen ('-')."), key);
+                   _("Invalid name “%s”: the last character may not be a "
+                     "hyphen (“-”)"), key);
       return FALSE;
     }
 
   if (i > 1024)
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                   _("invalid name '%s': maximum length is 1024"), key);
+                   _("Invalid name “%s”: maximum length is 1024"), key);
       return FALSE;
     }
 
@@ -878,6 +882,8 @@ schema_state_free (gpointer data)
 
   g_free (state->path);
   g_free (state->gettext_domain);
+  g_free (state->extends_name);
+  g_free (state->list_of);
   g_hash_table_unref (state->keys);
   g_slice_free (SchemaState, state);
 }
@@ -925,7 +931,7 @@ schema_state_add_key (SchemaState  *state,
     {
       g_set_error_literal (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
-                           _("cannot add keys to a 'list-of' schema"));
+                           _("Cannot add keys to a “list-of” schema"));
       return NULL;
     }
 
@@ -965,7 +971,7 @@ schema_state_add_key (SchemaState  *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_MISSING_ATTRIBUTE,
-                   _("exactly one of 'type', 'enum' or 'flags' must "
+                   _("Exactly one of “type”, “enum” or “flags” must "
                      "be specified as an attribute to <key>"));
       return NULL;
     }
@@ -999,7 +1005,7 @@ schema_state_add_key (SchemaState  *state,
         {
           g_set_error (error, G_MARKUP_ERROR,
                        G_MARKUP_ERROR_INVALID_CONTENT,
-                       _("invalid GVariant type string '%s'"), type_string);
+                       _("Invalid GVariant type string “%s”"), type_string);
           return NULL;
         }
 
@@ -1029,7 +1035,7 @@ schema_state_add_override (SchemaState  *state,
     {
       g_set_error_literal (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
-                           _("<override> given but schema isn't "
+                           _("<override> given but schema isn’t "
                              "extending anything"));
       return;
     }
@@ -1042,7 +1048,7 @@ schema_state_add_override (SchemaState  *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   _("no <key name='%s'> to override"), key);
+                   _("No <key name='%s'> to override"), key);
       return;
     }
 
@@ -1136,7 +1142,7 @@ parse_state_start_schema (ParseState  *state,
           g_set_error (error, G_MARKUP_ERROR,
                        G_MARKUP_ERROR_INVALID_CONTENT,
                        _("<schema id='%s'> extends not yet existing "
-                         "schema '%s'"), id, extends_name);
+                         "schema “%s”"), id, extends_name);
           return;
         }
     }
@@ -1152,14 +1158,14 @@ parse_state_start_schema (ParseState  *state,
           g_set_error (error, G_MARKUP_ERROR,
                        G_MARKUP_ERROR_INVALID_CONTENT,
                        _("<schema id='%s'> is list of not yet existing "
-                         "schema '%s'"), id, list_of);
+                         "schema “%s”"), id, list_of);
           return;
         }
 
       if (tmp->path)
         {
           g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                       _("Can not be a list of a schema with a path"));
+                       _("Cannot be a list of a schema with a path"));
           return;
         }
     }
@@ -1169,7 +1175,7 @@ parse_state_start_schema (ParseState  *state,
       if (extends->path)
         {
           g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                       _("Can not extend a schema with a path"));
+                       _("Cannot extend a schema with a path"));
           return;
         }
 
@@ -1190,8 +1196,8 @@ parse_state_start_schema (ParseState  *state,
               g_set_error (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
                            _("<schema id='%s' list-of='%s'> extends <schema "
-                             "id='%s' list-of='%s'> but '%s' does not "
-                             "extend '%s'"), id, list_of, extends_name,
+                             "id='%s' list-of='%s'> but “%s” does not "
+                             "extend “%s”"), id, list_of, extends_name,
                            extends->list_of, list_of, extends->list_of);
               return;
             }
@@ -1206,22 +1212,29 @@ parse_state_start_schema (ParseState  *state,
   if (path && !(g_str_has_prefix (path, "/") && g_str_has_suffix (path, "/")))
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                   _("a path, if given, must begin and end with a slash"));
+                   _("A path, if given, must begin and end with a slash"));
       return;
     }
 
   if (path && list_of && !g_str_has_suffix (path, ":/"))
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                   _("the path of a list must end with ':/'"));
+                   _("The path of a list must end with “:/”"));
       return;
     }
 
   if (path && (g_str_has_prefix (path, "/apps/") ||
                g_str_has_prefix (path, "/desktop/") ||
                g_str_has_prefix (path, "/system/")))
-    g_printerr ("warning: Schema '%s' has path '%s'.  Paths starting with "
-                "'/apps/', '/desktop/' or '/system/' are deprecated.\n", id, path);
+    {
+      gchar *message = NULL;
+      message = g_strdup_printf (_("Warning: Schema “%s” has path “%s”.  "
+                                   "Paths starting with "
+                                   "“/apps/”, “/desktop/” or “/system/” are deprecated."),
+                                 id, path);
+      g_printerr ("%s\n", message);
+      g_free (message);
+    }
 
   state->schema_state = schema_state_new (path, gettext_domain,
                                           extends, extends_name, list_of);
@@ -1511,7 +1524,7 @@ key_state_end (KeyState **state_ptr,
     {
       g_set_error_literal (error,
                            G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                           "element <default> is required in <key>");
+                           _("Element <default> is required in <key>"));
       return;
     }
 }
@@ -1601,7 +1614,7 @@ text (GMarkupParseContext  *context,
         if (!g_ascii_isspace (text[i]))
           {
             g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                         _("text may not appear inside <%s>"),
+                         _("Text may not appear inside <%s>"),
                          g_markup_parse_context_get_element (context));
             break;
           }
@@ -1667,8 +1680,13 @@ output_key (gpointer key,
 
   if (state->child_schema &&
       !g_hash_table_lookup (data->schema_table, state->child_schema))
-    g_printerr ("warning: undefined reference to <schema id='%s'/>\n",
-                state->child_schema);
+    {
+      gchar *message = NULL;
+      message = g_strdup_printf (_("Warning: undefined reference to <schema id='%s'/>"),
+                                 state->child_schema);
+      g_printerr ("%s\n", message);
+      g_free (message);
+    }
 }
 
 static void
