@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,7 +35,7 @@ G_BEGIN_DECLS
 /**
  * GSocketConnectable:
  *
- * Interface for objects that contain or generate #GSocketAddress<!-- -->es.
+ * Interface for objects that contain or generate #GSocketAddresses.
  */
 typedef struct _GSocketConnectableIface GSocketConnectableIface;
 
@@ -73,7 +73,7 @@ GLIB_AVAILABLE_IN_ALL
 GSocketAddressEnumerator *g_socket_connectable_proxy_enumerate (GSocketConnectable *connectable);
 
 GLIB_AVAILABLE_IN_2_48
-gchar                    *g_socket_connectable_to_string (GSocketConnectable *addr);
+gchar                    *g_socket_connectable_to_string (GSocketConnectable *connectable);
 
 G_END_DECLS
 
