@@ -341,8 +341,7 @@ test_expander (void)
       total_read += res;
     }
 
-  g_assert (total_read == n_written);
-  g_assert (memcmp (converted1, converted2, n_written)  == 0);
+  g_assert_cmpmem (converted1, n_written, converted2, total_read);
 
   g_converter_reset (expander);
 
@@ -371,10 +370,9 @@ test_expander (void)
 
   g_output_stream_close (cstream_out, NULL, NULL);
 
-  g_assert (g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (mem_out)) == n_written);
-  g_assert (memcmp (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (mem_out)),
-		    converted1,
-		    n_written)  == 0);
+  g_assert_cmpmem (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (mem_out)),
+                   g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (mem_out)),
+                   converted1, n_written);
 
   g_free (converted1);
   g_free (converted2);
@@ -433,8 +431,8 @@ test_compressor (void)
       total_read += res;
     }
 
-  g_assert (total_read == n_read - 1); /* Last 2 zeros are combined */
-  g_assert (memcmp (converted, unexpanded_data, total_read)  == 0);
+  /* "n_read - 1" because last 2 zeros are combined */
+  g_assert_cmpmem (unexpanded_data, n_read - 1, converted, total_read);
 
   g_object_unref (cstream);
 
@@ -461,10 +459,11 @@ test_compressor (void)
 
   g_output_stream_close (cstream_out, NULL, NULL);
 
-  g_assert (g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (mem_out)) == n_read - 1); /* Last 2 zeros are combined */
-  g_assert (memcmp (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (mem_out)),
-		    unexpanded_data,
-		    g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (mem_out)))  == 0);
+  /* "n_read - 1" because last 2 zeros are combined */
+  g_assert_cmpmem (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (mem_out)),
+                   g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (mem_out)),
+                   unexpanded_data,
+                   n_read - 1);
 
   g_object_unref (cstream_out);
 
@@ -717,8 +716,7 @@ test_converter_leftover (void)
       total_read += res;
   }
 
-  g_assert_cmpint (total_read, ==, LEFTOVER_BUFSIZE);
-  g_assert (memcmp (converted, orig, LEFTOVER_BUFSIZE)  == 0);
+  g_assert_cmpmem (orig, LEFTOVER_BUFSIZE, converted, total_read);
 
   g_object_unref (cstream);
   g_free (orig);
@@ -747,6 +745,8 @@ test_roundtrip (gconstpointer data)
   GConverter *compressor, *decompressor;
   GZlibCompressorFormat fmt;
   gint lvl;
+  GFileInfo *info;
+  GFileInfo *info2;
 
   g_test_bug ("619945");
 
@@ -759,6 +759,12 @@ test_roundtrip (gconstpointer data)
 
   ostream1 = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
   compressor = G_CONVERTER (g_zlib_compressor_new (test->format, test->level));
+  info = g_file_info_new ();
+  g_file_info_set_name (info, "foo");
+  g_object_set (compressor, "file-info", info, NULL);
+  info2 = g_zlib_compressor_get_file_info (G_ZLIB_COMPRESSOR (compressor));
+  g_assert (info == info2);
+  g_object_unref (info);
   costream1 = g_converter_output_stream_new (ostream1, compressor);
   g_assert (g_converter_output_stream_get_converter (G_CONVERTER_OUTPUT_STREAM (costream1)) == compressor);
 
@@ -787,10 +793,9 @@ test_roundtrip (gconstpointer data)
   g_output_stream_splice (ostream2, cistream1, 0, NULL, &error);
   g_assert_no_error (error);
 
-  g_assert_cmpuint (DATA_LENGTH * sizeof (guint32), ==,
-    g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (ostream2)));
-  g_assert (memcmp (data0, g_memory_output_stream_get_data (
-    G_MEMORY_OUTPUT_STREAM (ostream2)), DATA_LENGTH * sizeof (guint32)) == 0);
+  g_assert_cmpmem (data0, DATA_LENGTH * sizeof (guint32),
+                   g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (ostream2)),
+                   g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (ostream2)));
   g_object_unref (istream1);
   g_converter_reset (decompressor);
   g_object_get (decompressor, "format", &fmt, NULL);
@@ -1003,7 +1008,7 @@ test_converter_pollable (void)
 
       if (outptr < expanded_end)
 	{
-	  res = g_output_stream_write (socket_out,
+	   res = g_output_stream_write (socket_out,
 				       outptr,
 				       MIN (1000, (expanded_end - outptr)),
 				       NULL, &error);
@@ -1049,8 +1054,8 @@ test_converter_pollable (void)
       total_read += res;
     }
 
-  g_assert (total_read == n_read - 1); /* Last 2 zeros are combined */
-  g_assert (memcmp (converted, unexpanded_data, total_read)  == 0);
+  /* "n_read - 1" because last 2 zeros are combined */
+  g_assert_cmpmem (unexpanded_data, n_read - 1, converted, total_read);
 
   g_object_unref (cstream);
   g_object_unref (left);
@@ -1066,6 +1071,8 @@ test_converter_pollable (void)
   cstream_out = g_converter_output_stream_new (mem_out, compressor);
   g_object_unref (mem_out);
   pollable_out = G_POLLABLE_OUTPUT_STREAM (cstream_out);
+  g_assert (g_pollable_output_stream_can_poll (pollable_out));
+  g_assert (g_pollable_output_stream_is_writable (pollable_out));
 
   for (i = 0; i < expanded_size; i++)
     {
@@ -1084,10 +1091,11 @@ test_converter_pollable (void)
 
   g_output_stream_close (cstream_out, NULL, NULL);
 
-  g_assert (g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (mem_out)) == n_read - 1); /* Last 2 zeros are combined */
-  g_assert (memcmp (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (mem_out)),
-		    unexpanded_data,
-		    g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (mem_out)))  == 0);
+  /* "n_read - 1" because last 2 zeros are combined */
+  g_assert_cmpmem (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (mem_out)),
+                   g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (mem_out)),
+                   unexpanded_data,
+                   n_read - 1);
 
   g_object_unref (cstream_out);
 
@@ -1153,6 +1161,29 @@ test_truncation (gconstpointer data)
   g_free (data0);
 }
 
+static void
+test_converter_basics (void)
+{
+  GConverter *converter;
+  GError *error = NULL;
+  gchar *to;
+  gchar *from;
+
+  converter = (GConverter *)g_charset_converter_new ("utf-8", "latin1", &error);
+  g_assert_no_error (error);
+  g_object_get (converter,
+                "to-charset", &to,
+                "from-charset", &from,
+                NULL);
+
+  g_assert_cmpstr (to, ==, "utf-8");
+  g_assert_cmpstr (from, ==, "latin1");
+
+  g_free (to);
+  g_free (from);
+  g_object_unref (converter);
+}
+
 int
 main (int   argc,
       char *argv[])
@@ -1182,6 +1213,7 @@ main (int   argc,
 
   g_test_bug_base ("http://bugzilla.gnome.org/");
 
+  g_test_add_func ("/converter/basics", test_converter_basics);
   g_test_add_func ("/converter-input-stream/expander", test_expander);
   g_test_add_func ("/converter-input-stream/compressor", test_compressor);
 
