@@ -17,6 +17,8 @@
  * Author: Matthias Clasen
  */
 
+#include <locale.h>
+
 #include <glib/glib.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
@@ -346,9 +348,14 @@ static void
 test_extra_getters (void)
 {
   GDesktopAppInfo *appinfo;
+  const gchar *lang;
   gchar *s;
   gboolean b;
 
+  lang = setlocale (LC_ALL, NULL);
+  g_setenv ("LANGUAGE", "de_DE.UTF8", TRUE);
+  setlocale (LC_ALL, "");
+
   appinfo = g_desktop_app_info_new_from_filename (g_test_get_filename (G_TEST_DIST, "appinfo-test.desktop", NULL));
   g_assert (appinfo != NULL);
 
@@ -359,10 +366,24 @@ test_extra_getters (void)
   g_assert_cmpstr (s, ==, "appinfo-class");
   g_free (s);
 
+  s = g_desktop_app_info_get_locale_string (appinfo, "X-JunkFood");
+  g_assert_cmpstr (s, ==, "Bratwurst");
+  g_free (s);
+
+  g_setenv ("LANGUAGE", "sv_SV.UTF8", TRUE);
+  setlocale (LC_ALL, "");
+
+  s = g_desktop_app_info_get_locale_string (appinfo, "X-JunkFood");
+  g_assert_cmpstr (s, ==, "Burger"); /* fallback */
+  g_free (s);
+
   b = g_desktop_app_info_get_boolean (appinfo, "Terminal");
   g_assert (b);
 
   g_object_unref (appinfo);
+
+  g_setenv ("LANGUAGE", lang, TRUE);
+  setlocale (LC_ALL, "");
 }
 
 static void
@@ -605,7 +626,7 @@ assert_implementations (const gchar *interface,
 }
 
 #define ALL_USR_APPS  "evince-previewer.desktop nautilus-classic.desktop gnome-font-viewer.desktop "         \
-                      "baobab.desktop yelp.desktop eog.desktop cheese.desktop gnome-clocks.desktop "         \
+                      "baobab.desktop yelp.desktop eog.desktop cheese.desktop org.gnome.clocks.desktop "         \
                       "gnome-contacts.desktop kde4-kate.desktop gcr-prompter.desktop totem.desktop "         \
                       "gnome-terminal.desktop nautilus-autorun-software.desktop gcr-viewer.desktop "         \
                       "nautilus-connect-server.desktop kde4-dolphin.desktop gnome-music.desktop "            \
@@ -646,6 +667,17 @@ test_search (void)
   assert_search ("image viewer", "", FALSE, TRUE, NULL, NULL);
   assert_search ("image viewer", "", TRUE, TRUE, NULL, NULL);
 
+  /* There're "flatpak" apps (clocks) installed as well - they should *not*
+   * match the prefix command ("/bin/sh") in the Exec= line though.
+   */
+  assert_search ("sh", "gnome-terminal.desktop\n", TRUE, FALSE, NULL, NULL);
+
+  /* "frobnicator.desktop" is ignored by get_all() because the binary is
+   * missing, but search should still find it (to avoid either stale results
+   * from the cache or expensive stat() calls for each potential result)
+   */
+  assert_search ("frobni", "frobnicator.desktop\n", TRUE, FALSE, NULL, NULL);
+
   /* Obvious multi-word search */
   assert_search ("gno hel", "yelp.desktop\n", TRUE, TRUE, NULL, NULL);
 
@@ -664,7 +696,7 @@ test_search (void)
    * yelp and gnome-contacts, though.
    */
   assert_search ("gnome", "eog.desktop\n"
-                          "gnome-clocks.desktop\n"
+                          "org.gnome.clocks.desktop\n"
                           "yelp.desktop gnome-contacts.desktop\n", TRUE, TRUE, NULL, NULL);
 
   /* eog has exec name 'false' in usr only */
