@@ -220,7 +220,8 @@ g_array_sized_new (gboolean zero_terminated,
  *
  * The @clear_func will be called when an element in the array
  * data segment is removed and when the array is freed and data
- * segment is deallocated as well.
+ * segment is deallocated as well. @clear_func will be passed a
+ * pointer to the element to clear, rather than the element itself.
  *
  * Note that in contrast with other uses of #GDestroyNotify
  * functions, @clear_func is expected to clear the contents of
@@ -1102,9 +1103,21 @@ ptr_array_free (GPtrArray      *array,
 
   if (flags & FREE_SEGMENT)
     {
+      /* Data here is stolen and freed manually. It is an
+       * error to attempt to access the array data (including
+       * mutating the array bounds) during destruction).
+       *
+       * https://bugzilla.gnome.org/show_bug.cgi?id=769064
+       */
+      gpointer *stolen_pdata = g_steal_pointer (&rarray->pdata);
       if (rarray->element_free_func != NULL)
-        g_ptr_array_foreach (array, (GFunc) rarray->element_free_func, NULL);
-      g_free (rarray->pdata);
+        {
+          gsize i;
+          for (i = 0; i < rarray->len; ++i)
+            rarray->element_free_func (stolen_pdata[i]);
+        }
+
+      g_free (stolen_pdata);
       segment = NULL;
     }
   else
@@ -1157,6 +1170,7 @@ g_ptr_array_set_size  (GPtrArray *array,
   GRealPtrArray *rarray = (GRealPtrArray *)array;
 
   g_return_if_fail (rarray);
+  g_return_if_fail (rarray->len == 0 || (rarray->len != 0 && rarray->pdata != NULL));
 
   if (length > rarray->len)
     {
@@ -1177,6 +1191,39 @@ g_ptr_array_set_size  (GPtrArray *array,
   rarray->len = length;
 }
 
+static gpointer
+ptr_array_remove_index (GPtrArray *array,
+                        guint      index_,
+                        gboolean   fast,
+                        gboolean   free_element)
+{
+  GRealPtrArray *rarray = (GRealPtrArray *) array;
+  gpointer result;
+
+  g_return_val_if_fail (rarray, NULL);
+  g_return_val_if_fail (rarray->len == 0 || (rarray->len != 0 && rarray->pdata != NULL), NULL);
+
+  g_return_val_if_fail (index_ < rarray->len, NULL);
+
+  result = rarray->pdata[index_];
+
+  if (rarray->element_free_func != NULL && free_element)
+    rarray->element_free_func (rarray->pdata[index_]);
+
+  if (index_ != rarray->len - 1 && !fast)
+    memmove (rarray->pdata + index_, rarray->pdata + index_ + 1,
+             sizeof (gpointer) * (rarray->len - index_ - 1));
+  else if (index_ != rarray->len - 1)
+    rarray->pdata[index_] = rarray->pdata[rarray->len - 1];
+
+  rarray->len -= 1;
+
+  if (G_UNLIKELY (g_mem_gc_friendly))
+    rarray->pdata[rarray->len] = NULL;
+
+  return result;
+}
+
 /**
  * g_ptr_array_remove_index:
  * @array: a #GPtrArray
@@ -1185,36 +1232,16 @@ g_ptr_array_set_size  (GPtrArray *array,
  * Removes the pointer at the given index from the pointer array.
  * The following elements are moved down one place. If @array has
  * a non-%NULL #GDestroyNotify function it is called for the removed
- * element.
+ * element. If so, the return value from this function will potentially point
+ * to freed memory (depending on the #GDestroyNotify implementation).
  *
- * Returns: the pointer which was removed
+ * Returns: (nullable): the pointer which was removed
  */
 gpointer
 g_ptr_array_remove_index (GPtrArray *array,
                           guint      index_)
 {
-  GRealPtrArray *rarray = (GRealPtrArray *)array;
-  gpointer result;
-
-  g_return_val_if_fail (rarray, NULL);
-
-  g_return_val_if_fail (index_ < rarray->len, NULL);
-
-  result = rarray->pdata[index_];
-  
-  if (rarray->element_free_func != NULL)
-    rarray->element_free_func (rarray->pdata[index_]);
-
-  if (index_ != rarray->len - 1)
-    memmove (rarray->pdata + index_, rarray->pdata + index_ + 1,
-             sizeof (gpointer) * (rarray->len - index_ - 1));
-  
-  rarray->len -= 1;
-
-  if (G_UNLIKELY (g_mem_gc_friendly))
-    rarray->pdata[rarray->len] = NULL;
-
-  return result;
+  return ptr_array_remove_index (array, index_, FALSE, TRUE);
 }
 
 /**
@@ -1226,35 +1253,59 @@ g_ptr_array_remove_index (GPtrArray *array,
  * The last element in the array is used to fill in the space, so
  * this function does not preserve the order of the array. But it
  * is faster than g_ptr_array_remove_index(). If @array has a non-%NULL
- * #GDestroyNotify function it is called for the removed element.
+ * #GDestroyNotify function it is called for the removed element. If so, the
+ * return value from this function will potentially point to freed memory
+ * (depending on the #GDestroyNotify implementation).
  *
- * Returns: the pointer which was removed
+ * Returns: (nullable): the pointer which was removed
  */
 gpointer
 g_ptr_array_remove_index_fast (GPtrArray *array,
                                guint      index_)
 {
-  GRealPtrArray *rarray = (GRealPtrArray *)array;
-  gpointer result;
+  return ptr_array_remove_index (array, index_, TRUE, TRUE);
+}
 
-  g_return_val_if_fail (rarray, NULL);
+/**
+ * g_ptr_array_steal_index:
+ * @array: a #GPtrArray
+ * @index_: the index of the pointer to steal
+ *
+ * Removes the pointer at the given index from the pointer array.
+ * The following elements are moved down one place. The #GDestroyNotify for
+ * @array is *not* called on the removed element; ownership is transferred to
+ * the caller of this function.
+ *
+ * Returns: (transfer full) (nullable): the pointer which was removed
+ * Since: 2.58
+ */
+gpointer
+g_ptr_array_steal_index (GPtrArray *array,
+                         guint      index_)
+{
+  return ptr_array_remove_index (array, index_, FALSE, FALSE);
+}
 
-  g_return_val_if_fail (index_ < rarray->len, NULL);
-
-  result = rarray->pdata[index_];
-
-  if (rarray->element_free_func != NULL)
-    rarray->element_free_func (rarray->pdata[index_]);
-
-  if (index_ != rarray->len - 1)
-    rarray->pdata[index_] = rarray->pdata[rarray->len - 1];
-
-  rarray->len -= 1;
-
-  if (G_UNLIKELY (g_mem_gc_friendly))
-    rarray->pdata[rarray->len] = NULL;
-
-  return result;
+/**
+ * g_ptr_array_steal_index_fast:
+ * @array: a #GPtrArray
+ * @index_: the index of the pointer to steal
+ *
+ * Removes the pointer at the given index from the pointer array.
+ * The last element in the array is used to fill in the space, so
+ * this function does not preserve the order of the array. But it
+ * is faster than g_ptr_array_steal_index(). The #GDestroyNotify for @array is
+ * *not* called on the removed element; ownership is transferred to the caller
+ * of this function.
+ *
+ * Returns: (transfer full) (nullable): the pointer which was removed
+ * Since: 2.58
+ */
+gpointer
+g_ptr_array_steal_index_fast (GPtrArray *array,
+                              guint      index_)
+{
+  return ptr_array_remove_index (array, index_, TRUE, FALSE);
 }
 
 /**
@@ -1281,6 +1332,7 @@ g_ptr_array_remove_range (GPtrArray *array,
   guint n;
 
   g_return_val_if_fail (rarray != NULL, NULL);
+  g_return_val_if_fail (rarray->len == 0 || (rarray->len != 0 && rarray->pdata != NULL), NULL);
   g_return_val_if_fail (index_ <= rarray->len, NULL);
   g_return_val_if_fail (index_ + length <= rarray->len, NULL);
 
@@ -1331,6 +1383,7 @@ g_ptr_array_remove (GPtrArray *array,
   guint i;
 
   g_return_val_if_fail (array, FALSE);
+  g_return_val_if_fail (array->len == 0 || (array->len != 0 && array->pdata != NULL), FALSE);
 
   for (i = 0; i < array->len; i += 1)
     {
@@ -1368,6 +1421,7 @@ g_ptr_array_remove_fast (GPtrArray *array,
   guint i;
 
   g_return_val_if_fail (rarray, FALSE);
+  g_return_val_if_fail (rarray->len == 0 || (rarray->len != 0 && rarray->pdata != NULL), FALSE);
 
   for (i = 0; i < rarray->len; i += 1)
     {
@@ -1396,6 +1450,7 @@ g_ptr_array_add (GPtrArray *array,
   GRealPtrArray *rarray = (GRealPtrArray *)array;
 
   g_return_if_fail (rarray);
+  g_return_if_fail (rarray->len == 0 || (rarray->len != 0 && rarray->pdata != NULL));
 
   g_ptr_array_maybe_expand (rarray, 1);
 
