@@ -99,9 +99,9 @@ extern gboolean _g_main_poll_debug;
  * don't want to run the full main loop.
  *
  * Each element of @fds is a #GPollFD describing a single file
- * descriptor to poll. The %fd field indicates the file descriptor,
- * and the %events field indicates the events to poll for. On return,
- * the %revents fields will be filled with the events that actually
+ * descriptor to poll. The @fd field indicates the file descriptor,
+ * and the @events field indicates the events to poll for. On return,
+ * the @revents fields will be filled with the events that actually
  * occurred.
  *
  * On POSIX systems, the file descriptors in @fds can be any sort of
@@ -110,7 +110,7 @@ extern gboolean _g_main_poll_debug;
  * Windows, the easiest solution is to construct all of your
  * #GPollFDs with g_io_channel_win32_make_pollfd().
  *
- * Returns: the number of entries in @fds whose %revents fields
+ * Returns: the number of entries in @fds whose @revents fields
  * were filled in, or 0 if the operation timed out, or -1 on error or
  * if the call was interrupted.
  *
@@ -271,7 +271,7 @@ g_poll (GPollFD *fds,
         {
           if (nhandles == MAXIMUM_WAIT_OBJECTS)
             {
-              g_warning ("Too many handles to wait for!\n");
+              g_warning ("Too many handles to wait for!");
               break;
             }
           else
