@@ -544,7 +544,6 @@ do_spawn_with_pipes (gint                 *exit_status,
   GError *conv_error = NULL;
   gint conv_error_index;
   gchar *helper_process;
-  CONSOLE_CURSOR_INFO cursor_info;
   wchar_t *whelper, **wargv, **wenvp;
   gchar *glib_dll_directory;
 
@@ -1238,8 +1237,6 @@ g_spawn_check_exit_status (gint      exit_status,
   return ret;
 }
 
-#endif /* !GSPAWN_HELPER */
-
 #ifdef G_OS_WIN32
 
 /* Binary compatibility versions. Not for newly compiled code. */
@@ -1372,4 +1369,6 @@ g_spawn_command_line_async_utf8 (const gchar *command_line,
   return g_spawn_command_line_async (command_line, error);
 }
 
-#endif
+#endif /* G_OS_WIN32 */
+
+#endif /* !GSPAWN_HELPER */
