@@ -83,7 +83,7 @@ GLIB_AVAILABLE_IN_ALL
 const gchar * const * g_win32_get_system_data_dirs_for_module (void (*address_of_function)(void));
 #endif
 
-#if defined (G_OS_WIN32) && defined (G_CAN_INLINE)
+#if defined (G_OS_WIN32) && defined (G_CAN_INLINE) && !defined (__cplusplus)
 /* This function is not part of the public GLib API either. Just call
  * g_get_system_data_dirs() in your code, never mind that that is
  * actually a macro and you will in fact call this inline function.
@@ -191,7 +191,7 @@ gchar *g_format_size_full   (guint64          size,
 GLIB_AVAILABLE_IN_2_30
 gchar *g_format_size        (guint64          size);
 
-GLIB_DEPRECATED_FOR(g_format_size)
+GLIB_DEPRECATED_IN_2_30_FOR(g_format_size)
 gchar *g_format_size_for_display (goffset size);
 
 #ifndef G_DISABLE_DEPRECATED
