@@ -39,6 +39,64 @@ sum_up (gpointer data,
   *sum += GPOINTER_TO_INT (data);
 }
 
+/* Check that expanding an array with g_array_set_size() clears the new elements
+ * if @clear_ was specified during construction. */
+static void
+array_new_cleared (void)
+{
+  GArray *garray;
+  gsize i;
+
+  garray = g_array_new (FALSE, TRUE, sizeof (gint));
+  g_assert_cmpuint (garray->len, ==, 0);
+
+  g_array_set_size (garray, 5);
+  g_assert_cmpuint (garray->len, ==, 5);
+
+  for (i = 0; i < 5; i++)
+    g_assert_cmpint (g_array_index (garray, gint, i), ==, 0);
+
+  g_array_unref (garray);
+}
+
+/* As with array_new_cleared(), but with a sized array. */
+static void
+array_new_sized_cleared (void)
+{
+  GArray *garray;
+  gsize i;
+
+  garray = g_array_sized_new (FALSE, TRUE, sizeof (gint), 10);
+  g_assert_cmpuint (garray->len, ==, 0);
+
+  g_array_set_size (garray, 5);
+  g_assert_cmpuint (garray->len, ==, 5);
+
+  for (i = 0; i < 5; i++)
+    g_assert_cmpint (g_array_index (garray, gint, i), ==, 0);
+
+  g_array_unref (garray);
+}
+
+/* Check that a zero-terminated array does actually have a zero terminator. */
+static void
+array_new_zero_terminated (void)
+{
+  GArray *garray;
+  gchar *out_str = NULL;
+
+  garray = g_array_new (TRUE, FALSE, sizeof (gchar));
+  g_assert_cmpuint (garray->len, ==, 0);
+
+  g_array_append_vals (garray, "hello", strlen ("hello"));
+  g_assert_cmpuint (garray->len, ==, 5);
+  g_assert_cmpstr (garray->data, ==, "hello");
+
+  out_str = g_array_free (garray, FALSE);
+  g_assert_cmpstr (out_str, ==, "hello");
+  g_free (out_str);
+}
+
 static void
 array_append (void)
 {
@@ -600,6 +658,59 @@ pointer_array_find_non_empty (void)
   g_ptr_array_free (array, TRUE);
 }
 
+static void
+steal_destroy_notify (gpointer data)
+{
+  guint *counter = data;
+  *counter = *counter + 1;
+}
+
+/* Test that g_ptr_array_steal_index() and g_ptr_array_steal_index_fast() can
+ * remove elements from a pointer array without the #GDestroyNotify being called. */
+static void
+pointer_array_steal (void)
+{
+  guint i1 = 0, i2 = 0, i3 = 0, i4 = 0;
+  gpointer out1, out2;
+  GPtrArray *array = g_ptr_array_new_with_free_func (steal_destroy_notify);
+
+  g_ptr_array_add (array, &i1);
+  g_ptr_array_add (array, &i2);
+  g_ptr_array_add (array, &i3);
+  g_ptr_array_add (array, &i4);
+
+  g_assert_cmpuint (array->len, ==, 4);
+
+  /* Remove a single element. */
+  out1 = g_ptr_array_steal_index (array, 0);
+  g_assert_true (out1 == &i1);
+  g_assert_cmpuint (i1, ==, 0);  /* should not have been destroyed */
+
+  /* Following elements should have been moved down. */
+  g_assert_cmpuint (array->len, ==, 3);
+  g_assert_true (g_ptr_array_index (array, 0) == &i2);
+  g_assert_true (g_ptr_array_index (array, 1) == &i3);
+  g_assert_true (g_ptr_array_index (array, 2) == &i4);
+
+  /* Remove another element, quickly. */
+  out2 = g_ptr_array_steal_index_fast (array, 0);
+  g_assert_true (out2 == &i2);
+  g_assert_cmpuint (i2, ==, 0);  /* should not have been destroyed */
+
+  /* Last element should have been swapped in place. */
+  g_assert_cmpuint (array->len, ==, 2);
+  g_assert_true (g_ptr_array_index (array, 0) == &i4);
+  g_assert_true (g_ptr_array_index (array, 1) == &i3);
+
+  /* Check that destroying the pointer array doesn’t affect the stolen elements. */
+  g_ptr_array_unref (array);
+
+  g_assert_cmpuint (i1, ==, 0);
+  g_assert_cmpuint (i2, ==, 0);
+  g_assert_cmpuint (i3, ==, 1);
+  g_assert_cmpuint (i4, ==, 1);
+}
+
 static void
 byte_array_append (void)
 {
@@ -890,6 +1001,9 @@ main (int argc, char *argv[])
   g_test_bug_base ("https://bugzilla.gnome.org/");
 
   /* array tests */
+  g_test_add_func ("/array/new/cleared", array_new_cleared);
+  g_test_add_func ("/array/new/sized-cleared", array_new_sized_cleared);
+  g_test_add_func ("/array/new/zero-terminated", array_new_zero_terminated);
   g_test_add_func ("/array/append", array_append);
   g_test_add_func ("/array/prepend", array_prepend);
   g_test_add_func ("/array/remove", array_remove);
@@ -909,6 +1023,7 @@ main (int argc, char *argv[])
   g_test_add_func ("/pointerarray/sort-with-data", pointer_array_sort_with_data);
   g_test_add_func ("/pointerarray/find/empty", pointer_array_find_empty);
   g_test_add_func ("/pointerarray/find/non-empty", pointer_array_find_non_empty);
+  g_test_add_func ("/pointerarray/steal", pointer_array_steal);
 
   /* byte arrays */
   g_test_add_func ("/bytearray/append", byte_array_append);
