@@ -26,20 +26,21 @@
 #include <sys/time.h>
 #endif
 #include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_SYS_PRCTL_H
 #include <sys/prctl.h>
 #endif
 
-
 #include <glib.h>
 
-#ifndef G_OS_WIN32
+#ifdef G_OS_UNIX
+#include <unistd.h>
 #include <sys/resource.h>
 #endif
 
+#ifdef THREADS_POSIX
+#include <pthread.h>
+#endif
+
 static gpointer
 thread1_func (gpointer data)
 {
@@ -136,11 +137,17 @@ test_thread4 (void)
   GError *error;
   gint ret;
 
+  /* Linux CAP_SYS_RESOURCE overrides RLIMIT_NPROC, and probably similar
+   * things are true on other systems.
+   */
+  if (getuid () == 0 || geteuid () == 0)
+    return;
+
   getrlimit (RLIMIT_NPROC, &nl);
   nl.rlim_cur = 1;
 
   if ((ret = prlimit (getpid(), RLIMIT_NPROC, &nl, &ol)) != 0)
-    g_error ("prlimit failed: %s\n", g_strerror (ret));
+    g_error ("prlimit failed: %s", g_strerror (ret));
 
   error = NULL;
   thread = g_thread_try_new ("a", thread1_func, NULL, &error);
@@ -149,7 +156,7 @@ test_thread4 (void)
   g_error_free (error);
 
   if ((ret = prlimit (getpid (), RLIMIT_NPROC, &ol, NULL)) != 0)
-    g_error ("resetting RLIMIT_NPROC failed: %s\n", g_strerror (ret));
+    g_error ("resetting RLIMIT_NPROC failed: %s", g_strerror (ret));
 #endif
 }
 
@@ -167,14 +174,12 @@ test_thread5 (void)
 static gpointer
 thread6_func (gpointer data)
 {
-#ifdef HAVE_SYS_PRCTL_H
-#ifdef PR_GET_NAME
-  const gchar name[16];
+#if defined (HAVE_PTHREAD_SETNAME_NP_WITH_TID) && defined (HAVE_PTHREAD_GETNAME_NP)
+  char name[16];
 
-  prctl (PR_GET_NAME, name, 0, 0, 0, 0);
+  pthread_getname_np (pthread_self(), name, 16);
 
-  g_assert_cmpstr (name, ==, (gchar*)data);
-#endif
+  g_assert_cmpstr (name, ==, data);
 #endif
 
   return NULL;
